/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TCaptionedComponent - Type CaptionedComponent

Hirarchie:
TObject -> TComponent -> TCaptionedComponent
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TCaptionedComponant est la racine de la hirarchie des composants visuels
 * tiquetts.
 @since Release 1.0
*/

//Structure de l'objet de Type CaptionedComponent extension de TComponent
typedef struct CaptionedComponent {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct CaptionedComponent*	(*clone)		(struct CaptionedComponent*);
	BOOL												(*equals)		(struct CaptionedComponent*, struct Object*);
	void												(*finalize)	(struct CaptionedComponent*);
	TString* 										(*toString) (struct CaptionedComponent*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Component*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TString* caption;	//L'tiquette du composant
	void							(*setCaption)	(struct CaptionedComponent*, TString*);
} TCaptionedComponent;

//Dclaration des mthodes statiques
int CaptionedComponent_getFontHeight	();
int CaptionedComponent_getFontWidth		();

//Dclaration des constructeurs
TCaptionedComponent* TCaptionedComponent0();
TCaptionedComponent* TCaptionedComponent1(TString*);
TCaptionedComponent* TCaptionedComponent2(char*);

//Dclaration des mthodes
TCaptionedComponent*	TCaptionedComponent_clone				(TCaptionedComponent*);
BOOL									TCaptionedComponent_equals			(TCaptionedComponent*, TObject*);
void									TCaptionedComponent_finalize		(TCaptionedComponent*);
void									TCaptionedComponent_setCaption	(TCaptionedComponent*, TString*);
TString*							TCaptionedComponent_toString		(TCaptionedComponent*);

/**
 * Ce constructeur instancie un objet de Type CaptionedComponent.
 @since Release 1.0
*/
TCaptionedComponent* TCaptionedComponent0 () {
	TCaptionedComponent *this = (TCaptionedComponent*)malloc(sizeof(TCaptionedComponent));
	
	this->class  = "TCaptionedComponent";
	this->super  = "TComponent";
	
	this->height 	= 0;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->caption	= s("");
	
	this->clone 	 = TCaptionedComponent_clone;
	this->equals 	 = TCaptionedComponent_equals;
	this->finalize = TCaptionedComponent_finalize;
	this->toString = TCaptionedComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TComponent_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	this->setCaption = TCaptionedComponent_setCaption;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type CaptionedComponent en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TCaptionedComponent* TCaptionedComponent1 (TString* caption) {
	
	TCaptionedComponent *this = TCaptionedComponent0();
	
	this->setCaption(this, caption);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type CaptionedComponent en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TCaptionedComponent* TCaptionedComponent2 (char* caption) {
	
	TCaptionedComponent *this;
	TString *string;
	
	string = s(caption);
	
	this = TCaptionedComponent1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type CaptionedComponent qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TCaptionedComponent* TCaptionedComponent_clone(TCaptionedComponent* this) {

	TCaptionedComponent *clone;
	
	clone = TCaptionedComponent0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->caption	= c(this->caption);
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TCaptionedComponent_finalize (TCaptionedComponent* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->caption->finalize(this->caption);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Dans le cas d'un objet de Type CaptionedComponent, une comparaison est tablie
 * entre les reprsentations en chane de caractres des objets.
 @param object0 L'objet  comparer.
 @return result - Vrai si les deux objets sont gaux.
 @since Release 1.0
*/
BOOL TCaptionedComponent_equals (TCaptionedComponent* this, TObject* object0) {
	
	BOOL result;
	
	if (this->caption->equals(this->caption, object0->toString(object0))) {
		result = TRUE;
	}
	else {
		result = FALSE;
	}
	
	return result;
}

/**
 * Cette mthode change l'tiquette du composant.  Si les dimensions du
 * CaptionedComponent ne suffisent pas  la nouvelle tiquette, elles sont ajustes
 * automatiquement.
 @param caption L'tiquette du composant
 @since Release 1.0
*/
void TCaptionedComponent_setCaption(TCaptionedComponent* this, TString* caption) {

	BOOL visible = this->visible;
	
	int cptH;
	int cptW;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	cptW = (caption->length(caption) + 1) * CaptionedComponent_getFontWidth();
	cptH = CaptionedComponent_getFontHeight();
	
	if (this->width < cptW) {
		this->width = cptW + 2;
	}
	
	if (this->height < cptH) {
		this->height = cptH + 2;
	}
	
	this->caption->finalize(this->caption);
	this->caption = c(caption);
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode retourne une chane de caractres reprsentant l'objet.
 * Pour la classe TCaptionedComponent, il s'agit de l'tiquette.
 @return string - La chane de caractre reprsentant l'objet.
 @since Release 1.0
*/
TString* TCaptionedComponent_toString	(TCaptionedComponent* this) {
	
	TString* string;
	
	string = c(this->caption);
	
	return string;
}

/**
 * Cette mthode retourne la hauteur de la police utilise par le systme.
 @return La hauteur de la police
 @since Release 1.0
*/
int CaptionedComponent_getFontHeight () {
	
	int cptH = 0;
	
	switch (FontGetSys()) {
		case F_4x6: {
			cptH = 6;
			break;
		}
		case F_6x8: {
			cptH = 8;
			break;
		}
		case F_8x10: {
			cptH = 10;
			break;
		}
	}
	
	return cptH;
}

/**
 * Cette mthode retourne la largeur de la police utilise par le systme.
 @return La largeur de la police
 @since Release 1.0
*/
int CaptionedComponent_getFontWidth () {

	int cptW = 0;
	
	switch (FontGetSys()) {
		case F_4x6: {
			cptW = 4;
			break;
		}
		case F_6x8: {
			cptW = 6;
			break;
		}
		case F_8x10: {
			cptW = 8;
			break;
		}
	}
	
	return cptW;	
}
