/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TImage - Type Image

Hirarchie:
TObject -> TComponent -> TImage
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TImage permet d'afficher des images.
 @since Release 1.0
*/

//Structure de l'objet de Type Image extension de TComponent
typedef struct Image {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Image*			(*clone)		(struct Image*);
	BOOL							(*equals)		(struct Image*, struct Image*);
	void							(*finalize)	(struct Image*);
	TString* 					(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Image*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	BITMAP *content; //Le contenu de l'image
	void							(*loadVAR)		(struct Image*, TString*, int, int);
	void							(*setContent)	(struct Image*, BITMAP*);
} TImage;

//Dclaration des constructeurs
TImage* TImage0();
TImage* TImage1(BITMAP*);
TImage* TImage2(TString*, int, int);

//Dclaration des mthodes
TImage*			TImage_clone				(TImage*);
BOOL				TImage_equals				(TImage*, TImage*);
void				TImage_finalize			(TImage*);
void				TImage_print				(TImage*);
void				TImage_loadVAR			(TImage*, TString*, int, int);
void				TImage_setContent		(TImage*, BITMAP*);

//Dclaration des mthodes statiques
void Image_showPicvar (char*, int, int, int);

/**
 * Ce constructeur instancie un objet de Type Image.
 @since Release 1.0
*/
TImage* TImage0 () {
	TImage *this = (TImage*)malloc(sizeof(TImage));
	
	this->class  = "TImage";
	this->super  = "TComponent";
	
	this->height 	= 0;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->content	= (BITMAP*)malloc(sizeof(BITMAP));
	
	this->clone 	 = TImage_clone;
	this->equals 	 = TImage_equals;
	this->finalize = TImage_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TImage_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	this->setContent = TImage_setContent;
	this->loadVAR		 = TImage_loadVAR;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type Image en
 * spcifiant le contenu.
 @param image Le bitmap
 @since Release 1.0
*/
TImage* TImage1 (BITMAP* image) {
	
	TImage *this = TImage0();
	
	this->setContent(this, image);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type Image en
 * spcifiant le contenu  l'aide d'une variable TIOS.
 @param image La variable TIOS contenant l'image
 @since Release 1.0
*/
TImage* TImage2 (TString* image, int width, int height) {
	
	TImage *this = TImage0();
	
	this->loadVAR(this, image, width, height);
	
	return this;
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TImage_print(TImage* this) {
	
	BitmapPut (this->x, this->y, this->content, ScrRect, A_REPLACE);
}

/**
 * Cette mthode retourne un objet de Type Image qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TImage* TImage_clone(TImage* this) {

	TImage *clone;
	
	clone = TImage0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	
	clone->setContent(clone, this->content);
	
	return clone;
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Dans le cas d'un objet de Type Image, une comparaison est tablie
 * entre les contenus des images.
 @param image0 L'objet  comparer.
 @return result - Vrai si les deux objets sont gaux.
 @since Release 1.0
*/
BOOL TImage_equals (TImage* this, TImage* image0) {

	BOOL result = FALSE;
	int i;

	if ( this->height == image0->height &&
			 this->width == image0->width
		 ) {
		result = TRUE;
		for (i=0; i<((this->height * this->width) / 8); i++) {
			if (this->content->Data[i] != image0->content->Data[i]) {
				result = FALSE;
			}
		}
	}
	
	return result;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TImage_finalize (TImage* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		free(this->img);
		free(this->content);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode change l'image du composant
 @param image L'image du composant
 @since Release 1.0
*/
void TImage_setContent(TImage* this, BITMAP* image) {

	BOOL visible = this->visible;
	int i;
	
	SCR_RECT rec;
	rec.xy.x0 = 0;
	rec.xy.y0 = 0;
	rec.xy.x1 = image->NumCols - 1;
	rec.xy.y1 = image->NumRows - 1;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->height = image->NumRows;
	this->width  = image->NumCols;
	
	free(this->content);
	
	this->content	= (BITMAP*)malloc(BitmapSize(&rec));

	//memcpy(this->content, image, BitmapSize(&rec));
	
	for (i = 0; i < BitmapSize(&rec); i++)
	{
		((char*)this->content)[i] = ((char*)image)[i];
	}
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode change l'image du composant  partir 
 * d'une variable TIOS (elle est convertie en BITMAP).
 @param image La variable TIOS
 @since Release 1.0
*/
void TImage_loadVAR(TImage* this, TString* image, int width, int height) {
	
	SCR_RECT rec;
	void *virtual = malloc (LCD_SIZE);
	BITMAP* bitmap;
	char *name;
	int i;
	
	PortSet (virtual, LCD_WIDTH - 1, LCD_HEIGHT - 1);
	
	name = File_stringToVar(image);
	
	Image_showPicvar(name, 0, 0, A_REPLACE);
	
	rec.xy.x0 = 0;
	rec.xy.y0 = 0;
	rec.xy.x1 = width - 1;
	rec.xy.y1 = height - 1;
	bitmap = (BITMAP*)malloc(BitmapSize(&rec));
	BitmapGet (&rec, bitmap);
	
	PortRestore();
	
	this->setContent(this, bitmap);
	
	free(name - image->length(image) - 1);
	free(bitmap);
	free(virtual);
}
/**
 * Cette mthode permet d'afficher une variable TIOS  l'cran.
 @param SymName Le nom de la variable TIOS
 @param length Le nombre de caractres de la variable
 @param x La position horizontale o afficher
 @param y La position verticale o afficher
 @param Attr L'attribut d'affichage
 @since Release 1.0
*/
void Image_showPicvar (char *SymName, int x, int y, int Attr) {
	SYM_ENTRY *sym_entry = SymFindPtr (SymName, 0);
	if (!sym_entry) return;
	if (peek (HToESI (sym_entry->handle)) != PIC_TAG) return;
	BitmapPut (x, y, HeapDeref (sym_entry->handle) + 2, ScrRect, Attr);
} 