/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TString - Type String

Hirarchie:
TObject -> TString
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TString reprsente une chane de caractre.  Il est recommend d'instancier tous les
 * littraux (ex: "abc") en tant qu'objets de Type String pour en faciliter la manipulation.
  @since Release 1.0
*/

//Structure de l'objet de Type String extension de TObject
typedef struct String {
	char* class;  //Le nom de la classe
	char* super;  //Le nom de la superclasse
	struct String* 	(*clone) 		(struct String*);
	BOOL 						(*equals) 	(struct String*, struct String*);
	void 						(*finalize)	(struct String*);
	struct String*	(*toString)	(struct String*);
	char* string; //La chaine de caractre (format ANSI)
	char 						(*charAt)						(struct String*, long);	
	BOOL 						(*compareTo) 				(struct String*, struct String*);
	struct String*	(*concat)						(struct String*, struct String*);
	BOOL 						(*endsWith)					(struct String*, struct String*);	
	BOOL 						(*equalsIgnoreCase)	(struct String*, struct String*);	
	void 						(*getChars)					(struct String*, long, long, char*, long);
	long 						(*indexOf0)					(struct String*, struct String*, long);
	long 						(*indexOf1) 				(struct String*, struct String*);
	long 						(*length) 					(struct String*);
	struct String* 	(*toLowerCase) 			(struct String*);
	struct String* 	(*toUpperCase) 			(struct String*);
	long						(*lastIndexOf0)			(struct String*, struct String*, long);
	long						(*lastIndexOf1)			(struct String*, struct String*);
	BOOL						(*regionMatches0)		(struct String*, BOOL, long, struct String*, long, long);
	BOOL						(*regionMatches1)		(struct String*, long, struct String*, long, long);
	struct String*	(*replace)					(struct String*, char, char);
	BOOL						(*startsWith0)			(struct String*, struct String*, long);
	BOOL						(*startsWith1)			(struct String*, struct String*);
	struct String*	(*substring0)				(struct String*, long, long);
	struct String*	(*substring1)				(struct String*, long);
	char*						(*toCharArray)			(struct String*);
	struct String*	(*trim)							(struct String*);
} TString;

//Dclaration des mthodes statiques
char* 		String_concatenateString 		(char*, char*);
char* 		String_copyValueOf0 				(char*, long, long);
char* 		String_copyValueOf1 				(char*, long);
char* 		String_duplicateString			(char*);
char* 		String_generateEmptyString 	(long);
TString*	String_valueOf0 						(char*, long, long);
TString*	String_valueOf1 						(char*, long);

//Dclaration des constructeurs
TString* TString0 ();
TString* TString1 (TString*);
TString* TString2 (long, ...);
TString* TString3 (char*);
TString* TString4 (long, ...);
TString* TString5 (char*, long);

//Dclaration des mthodes
char 			TString_charAt 						(TString*, long);
TString* 	TString_clone 						(TString*);
BOOL 			TString_compareTo 				(TString*, TString*);
TString*	TString_concat						(TString*, TString*);
BOOL 			TString_endsWith					(TString*, TString*);
BOOL 			TString_equals 						(TString*, TString*);
BOOL 			TString_equalsIgnoreCase 	(TString*, TString*);
void 			TString_finalize 					(TString*);
void 			TString_getChars 					(TString*, long, long, char*, long);
long 			TString_indexOf0 					(TString*, TString*, long);
long 			TString_indexOf1 					(TString*, TString*);
long 			TString_length 						(TString*);
TString* 	TString_toLowerCase 			(TString*);
TString* 	TString_toUpperCase 			(TString*);
long			TString_lastIndexOf0			(TString*, TString*, long);
long			TString_lastIndexOf1			(TString*, TString*);
BOOL			TString_regionMatches0		(TString*, BOOL, long, TString*, long, long);
BOOL			TString_regionMatches1		(TString*, long, TString*, long, long);
TString*	TString_replace						(TString*, char, char);
BOOL			TString_startsWith0				(TString*, TString*, long);
BOOL			TString_startsWith1				(TString*, TString*);
TString*	TString_substring0				(TString*, long, long);
TString*	TString_substring1				(TString*, long);
char*			TString_toCharArray				(TString*);
TString*	TString_toString					(TString*);
TString*	TString_trim							(TString*);

/**
 * Cette mthode gnre une chane de caractres vide ANSI.
 @param length Taille de la chane de caractres.
 @return string - La chane de caractres.
 @since Release 1.0
*/
char* String_generateEmptyString (long length) {
	char *string;
	
	string = (char *)malloc(sizeof(char) * (length + 1));
	
	string[length] = '\0';
	
	return string;
}

/**
 * Cette mthode concatne deux chanes de caractres ANSI.
 @param string0 La premire chane de caractres.
 @param string1 La deuxime chane de caractres.
 @return string - La chane de caractres concatne.
 @since Release 1.0
*/
char* String_concatenateString (char* string0, char* string1) {
	char *string;
	
	string = String_generateEmptyString(strlen(string0) + strlen(string1));
	
	strcpy(string, string0);
	
	strcat(string, string1);
	
	return string;
}

/**
 * Cette mthode copie une chane de caractres ANSI.
 @param string La chane de caractre  copier.
 @return copy - La copie de la chane.
 @since Release 1.0
*/
char* String_duplicateString (char* string) {
	char *copy;
	
	copy = String_generateEmptyString(strlen(string));
	
	strcpy(copy, string);
	
	return copy;
}

/**
 * Cette mthode gnre une chane de caractres ANSI  partir d'un
 * vecteur de caractres.
 @param charArray Le vecteur de caractres.
 @param offset L'index du premier caractre  copier.
 @param count Le nombre de caractres  copier.
 @return string - La chane de caractres.
 @since Release 1.0
*/
char* String_copyValueOf0 (char* charArray, long offset, long count) {
	long i;
	char *string;

	string = String_generateEmptyString(count);

	for (i=offset; i<(count + offset); i++) {
		string[i] = charArray[i];
	}

	return string;
}

/**
 * Cette mthode gnre une chane de caractres ANSI  partir d'un
 * vecteur de caractres.
 @param charArray Le vecteur de caractres.
 @param length Le nombre de caractres  copier.
 @return string - La chane de caractres.
 @since Release 1.0
*/
char* String_copyValueOf1 (char* charArray, long length) {
	char *string;

	string = String_copyValueOf0(charArray, 0, length);

	return string;
}

/**
 * Cette mthode gnre un objet de Type String  partir d'un vecteur de caractres.
 @param charArray Le vecteur de caractres.
 @param offset L'index du premier caractre  copier.
 @param count Le nombre de caractres  copier.
 @return string - La chane de caractres.
 @since Release 1.0
*/
TString* String_valueOf0 (char* charArray, long offset, long count) {
	long i;
	char *ANSIString;
	TString *string;

	ANSIString = String_generateEmptyString(count);

	for (i=offset; i<(count + offset); i++) {
		ANSIString[i] = charArray[i];
	}

	string = s(ANSIString);

	return string;
}

/**
 * Cette mthode gnre un objet de Type String  partir d'un vecteur de caractres.
 @param charArray Le vecteur de caractres.
 @param length La taille du vecteur.
 @return string - La chane de caractres.
 @since Release 1.0
*/
TString* String_valueOf1 (char* charArray, long length) {
	TString *string;

	string = String_valueOf0(charArray, 0, length);

	return string;
}

/**
 * Ce constructeur instancie un objet de Type String.
 @since Release 1.0
*/
TString* TString0 () {
	TString *this = (TString*)malloc(sizeof(TString));

	this->class 	= "TString";
	this->super 	= "TObject";
	
	this->string  = String_duplicateString("");

	this->clone    = TString_clone;
	this->equals   = TString_equals;
	this->finalize = TString_finalize;
	this->toString = TString_toString;
	
	this->charAt  					= TString_charAt;
	this->compareTo  				= TString_compareTo;
	this->concat  					= TString_concat;
	this->endsWith  				= TString_endsWith;
	this->equalsIgnoreCase  = TString_equalsIgnoreCase;
	this->getChars  				= TString_getChars;
	this->indexOf0  				= TString_indexOf0;
	this->indexOf1  				= TString_indexOf1;
	this->length  					= TString_length;
	this->toLowerCase  			= TString_toLowerCase;
	this->toUpperCase  			= TString_toUpperCase;
	this->lastIndexOf0  		= TString_lastIndexOf0;
	this->lastIndexOf1  		= TString_lastIndexOf1;
	this->regionMatches0  	= TString_regionMatches0;
	this->regionMatches1		= TString_regionMatches1;
	this->replace  					= TString_replace;
	this->startsWith0  			= TString_startsWith0;
	this->startsWith1  			= TString_startsWith1;
	this->substring0  			= TString_substring0;
	this->substring1  			= TString_substring1;
	this->toCharArray 			= TString_toCharArray;
	this->trim  						= TString_trim;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type String  partir d'un autre objet de Type String.
 @param string0 L'objet utilis pour initialiser la chane de caractres (avec la valeur que retournera toString).
 @since Release 1.0
*/
TString* TString1 (TString* string) {
	TString *this;
	
	this = TString0();
	
	this->string = String_duplicateString(string->string);
	
	return this; 
}

/**
 * Ce constructeur instancie un objet de Type String  partir d'un groupe d'objets.
 @param num Le nombre d'objets utiliss pour initialiser la chane de caractres.
 @since Release 1.0
*/
TString* TString2 (long num, ...) {
	TString *this;
	
	long i;
	TString *str;
	char *string;
	va_list argptr;
	void *freePtr;
	
	this = TString0 ();
	
	va_start(argptr, num);
	
	for (i=0; i<num; i++) {
		str = (TString*)(va_arg(argptr, void*));
		str = str->toString(str);
		string = str->string;
		freePtr = this->string;
		this->string = String_concatenateString(this->string, string);
		str->finalize(str);
		free(freePtr);
	}
	
	va_end(arg_ptr);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type String  partir d'une chane de caractres.
 @param string La chane de caractres ANSI.
 @since Release 1.0
*/
TString* TString3 (char* string) {
	TString *this;
	
	this = TString0();
	
	this->string = String_duplicateString(string);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type String  partir d'un groupe de chaines de
 * caractres ANSI.
 @param num Le nombre de chanes de caractres utilises.
 @since Release 1.0
*/
TString* TString4 (long num, ...) {
	TString *this;

	long i;
	char *string;
	va_list argptr;	
	void *freePtr;
	
	this = TString0 ();
	
	va_start(argptr, num);
	
	for (i=0; i<num; i++) {
		string = va_arg(argptr, char*);
		freePtr = this->string;
		this->string = String_concatenateString(this->string, string);
		free(freePtr);
	}
	
	va_end(arg_ptr);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type String  partir d'un vecteur de caractres.
 @param charArray Le vecteur de caractres.
 @param length Le nombre de caractres  copier.
 @since Release 1.0
*/
TString* TString5 (char* charArray, long length) {
	TString *this;
	
	this = s(String_copyValueOf1(charArray, length));
	
	return this;
}

/**
 * Cette mthode retourne une chane de caractres reprsentant l'objet.
 * Dans le cas d'un objet de Type String, elle se retourne elle-mme.
 @return string - La chane de caractre reprsentant l'objet.
 @since Release 1.0
*/
TString* TString_toString (TString* this) {
	
	TString *string;
	
	string = this->clone(this);
	
	return string;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TString_finalize (TString* this) {
	
	free(this->string);
	
	TRY
		free(this);
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode retourne la longueur de la chane de caractre.
 @return length - La longueur de la chane de caractres.
 @since Release 1.0
*/
long TString_length (TString* this) {

	long length;
	
	length = strlen(this->string);
	
	return length;
}

/**
 * Cette mthode compare deux chanes de caractres lexicographiquement.
 @param anotherString La chane  comparer.
 @return result - Vrai si les deux chanes sont lexicographiquement identiques.
 @since Release 1.0
*/
BOOL TString_compareTo (TString* this, TString* string) {

	long i;
	long length  = this->length(this);
	long length0 = string->length(string);
	BOOL result = FALSE;
	
	if (length == length0) {
		result = TRUE;
		for (i=0; i<length && result; i++) {
			if (this->string[i] != string->string[i]) {
				result = FALSE;
			}
		}
	}
	else {
		result = FALSE;
	}
	
	return result;
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Dans le cas d'un objet de Type String, la comparaison se fait lexicographiquement.
 @param anotherString L'objet  comparer.
 @return result - Vrai si les deux chanes de caractres sont lexicographiquement identiques.
 @since Release 1.0
*/
BOOL TString_equals (TString* this, TString* anotherString) {

	BOOL result;
	TString* str;
	
	str = anotherString->toString(anotherString);
	
	result = this->compareTo(this, str);
	
	str->finalize(str);
	
	return result;
}

/**
 * Cette mthode retourne un objet de Type String qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TString* TString_clone (TString* this) {

	TString *clone;
	
	clone = c(this);
	
	return clone;
}

/**
 * Cette mthode retourne le caractre  l'index spcifi.  L'index varie de 0  length() - 1.
 @param index L'index.
 @return character - Le caractre  l'index spcifi(NULL si l'index est invalide).
 @since Release 1.0
*/
char TString_charAt (TString* this, long index) {
	
	char character = -1;
	
	if (index < this->length(this) && index >= 0) {
		character = this->string[index];
	}
	
	return character;
}

/**
 * Cette mthode concatne deux objets de Type String.
 @param anotherString La chane  concatner.
 @return string - La nouvelle chane.
 @since Release 1.0
*/
TString* TString_concat (TString* this, TString* string) {
	
	TString *newString;
	
	newString = TString0();
	
	newString->string = String_concatenateString(this->string, string->string);
	
	return newString;
}

/**
 * Cette mthode retourne l'index de la premire occurence de la chane spcifie dans la chane
 * de caractres  partir de l'index spcifi.
 @param anotherString La chane de caractres.
 @param index L'index d'o commencer.
 @return result - L'index de la premire occurence de la chane de caractres spcifie.
 @since Release 1.0
*/
long TString_indexOf0 (TString* this, TString* string, long index) {

	BOOL trouve = FALSE;
	BOOL ok;
	long result = -1;
	long i;
	long n;
	
	if (index < this->length(this) && index >= 0) {
		for (i=index; i<this->length(this) && !trouve; i++) {
			if (this->string[i] == string->string[0]) {
				if (string->length(string) <= this->length(this) - i) {
					ok = TRUE;
					for (n=i; n<(string->length(string) + i) && ok; n++) {
						if (this->string[n] != string->string[n - i]) {
							ok = FALSE;
						}
					}
					if (ok) {
						trouve = TRUE;
						result = i;
					}
				}
			}
		}
	}
	
	return result;
}

/**
 * Cette mthode retourne l'index de la premire occurence de la chane spcifie dans la chane
 * de caractres.
 @param anotherString La chane de caractres.
 @return result - L'index de la premire occurence de la chane de caractres spcifie.
 @since Release 1.0
*/
long TString_indexOf1 (TString* this, TString* string) {
	
	long result;
	
	result = this->indexOf0(this, string, 0);
	
	return result;
}

/**
 * Cette mthode vrifie si la chane de caractres se termine avec le suffixe spcifi.
 @param suffix Le sufixe.
 @return result - Vrai si la chane de caractres se termine avec le suffixe spcifi.
 @since Release 1.0
*/
BOOL TString_endsWith (TString* this, TString* string) {

	TString *stringEnd;

	BOOL result;
	long end   = this->length(this);
	long begin = end - string->length(string);
	
	stringEnd = s(String_copyValueOf0(this->string, begin, end));
	
	result = stringEnd->compareTo(stringEnd, string);

	stringEnd->finalize(stringEnd);

	return result;
}

/**
 * Cette mthode change la case de la chane de caractres pour des minuscules.
 @return string - La chane de caractres en minuscule.
 @since Release 1.0
*/
TString* TString_toLowerCase (TString* this) {

	long i;
	TString *string;
	
	string = this->clone(this);
	
	for(i=0; i<this->length(this); i++)
	{
		if (string->string[i] > 'A' && string->string[i] < 'Z')
		{
			string->string[i] = string->string[i] + 32;
		}
	}
	
	return string;
}

/**
 * Cette mthode change la case de la chane de caractres pour des majuscules.
 @return string - La chane de caractres en majuscule.
 @since Release 1.0
*/
TString* TString_toUpperCase (TString* this) {

	long i;
	TString *string;
	
	string = this->clone(this);
	
	for(i=0; i<this->length(this); i++)
	{
		if (string->string[i] > 'a' && string->string[i] < 'z')
		{
			string->string[i] = string->string[i] - 32;
		}
	}
	
	return string;
}

/**
 * Cette mthode compare lexicographiquement une rgion de la chane de caractres avec 
 * celle d'une autre chane de caractres.
 @param ignoreCase Spcifie s'il faut ignorer la case.
 @param toffset L'index du dbut de la rgion  comparer de cette chane.
 @param anotherString La chane  comparer.
 @param ooffset L'index du dbut de la rgion  comparer de l'autre chane.
 @param length La taille des rgions  comparer.
 @return result - Vrai si les rgions sont lexicographiquement identiques.
 @since Release 1.0
*/
BOOL TString_regionMatches0 (TString* this, BOOL ignoreCase, long toffset, TString* string, long ooffset, long length) {
	
	BOOL result;
	TString *string0;
	TString *string1;
	
	string0 = this->substring0(this, toffset, toffset + length - 1);
	string1 = string->substring0(string, ooffset, ooffset + length - 1);
	
	if (ignoreCase) {
		result = string0->equalsIgnoreCase(string0, string1);
	}
	else {
		result = string0->compareTo(string0, string1);
	}
	
	string0->finalize(string0);
	string1->finalize(string1);
	
	return result;
}

/**
 * Cette mthode compare lexicographiquement une rgion de la chane de caractres avec 
 * celle d'une autre chane de caractres.
 @param toffset L'index du dbut de la rgion  comparer de cette chane.
 @param anotherString La chane  comparer.
 @param ooffset L'index du dbut de la rgion  comparer de l'autre chane.
 @param length La taille des rgions  comparer.
 @return result - Vrai si les rgions sont lexicographiquement identiques.
 @since Release 1.0
*/
BOOL TString_regionMatches1 (TString* this, long toffset, TString* anotherString, long ooffset, long length) {
	
	BOOL result;
	
	result = this->regionMatches0(this, FALSE, toffset, anotherString, ooffset, length);
	
	return result;
}

/**
 * Cette mthode compare lexicographiquement la chane de caractres avec une autre
 * chane de caractres, mais en ignorant la case.
 @param anotherString La chane  comparer.
 @return result - Vrai si les chanes sont lexicographiquement identiques.
 @since Release 1.0
*/
BOOL TString_equalsIgnoreCase (TString* this, TString* string) {
	
	BOOL result;
	
	TString *stringLow0 = this->toLowerCase(this);
	TString *stringLow1 = string->toLowerCase(string);
	
	result = stringLow0->compareTo(stringLow0, stringLow1);
	
	stringLow0->finalize(stringLow0);
	stringLow1->finalize(stringLow1);
	
	return result;
}

/**
 * Cette mthode copie les caractres de la chane de caractre vers un vecteur de caractres.
 @param srcBegin Index du premier caractre  copier.
 @param srcEnd Index du dernier caractre  copier.
 @param dst Vecteur de caractres de destination.
 @param dstBegin Index du sous-vecteur de caractres.
*/
void TString_getChars (TString* this, long srcBegin, long srcEnd, char* dst, long dstBegin) {
	
	long i;
	
	for (i=srcBegin; i<=srcEnd; i++) {
		dst[i-srcBegin+dstBegin] = this->string[i];
	}
}

/**
 * Cette mthode retourne l'index de la dernire occurence de la chane spcifie dans la chane
 * de caractres cherchant  reculon  partir de l'index spcifi.
 @param anotherString La chane de caractres.
 @param index L'index d'o commencer.
 @return result - L'index de la dernire occurence de la chane de caractres spcifie.
 @since Release 1.0
*/
long TString_lastIndexOf0 (TString* this, TString* string, long index) {

	BOOL trouve = FALSE;
	BOOL ok;
	long result = -1;
	long i;
	long n;

	if (index < this->length(this) && index >= 0) {
		for (i=index; i>=0 && !trouve; i--) {
			if (this->string[i] == string->string[0]) {
				if (string->length(string) <= index - i) {
					ok = TRUE;
					for (n=i; n<(string->length(string) + i) && ok; n++) {
						if (this->string[n] != string->string[n - i]) {
							ok = FALSE;
						}
					}
					if (ok) {
						trouve = TRUE;
						result = i;
					}
				}
			}
		}
	}

	return result;
}

/**
 * Cette mthode retourne l'index de la dernire occurence de la chane spcifie dans la chane
 * de caractres cherchant  reculon  partir de l'index spcifi.
 @param anotherString La chane de caractres.
 @param index L'index d'o commencer.
 @return result - L'index de la dernire occurence de la chane de caractres spcifie.
 @since Release 1.0
*/
long TString_lastIndexOf1 (TString* this, TString* string) {

	long result;

	result = this->lastIndexOf0(this, string, this->length(this) - 1);

	return result;
}

/**
 * Cette mthode remplace toutes les occurences du caractre spcifi dans la chane de
 * caractres par un caractre spcifi.
 @param oldChar Le caractre  remplacer.
 @param newChar Le caractre remplaant.
 @return string - La chane de caractres modifie.
 @since Release 1.0
*/
TString* TString_replace (TString* this, char oldChar, char newChar) {
	
	long i;
	TString *string = this->clone(this);
	
	for (i=0; i<this->length(this); i++) {
		if (this->string[i] == oldChar) {
			this->string[i] = newChar;
		}
	}
	
	return string;
}

/**
 * Cette mthode vrifie si la chane de caractres commence avec le prfixe spcifi.
 @param prefix Le prfixe.
 @param offset D'o commencer.
 @return result - Vrai si la chane de caractres commence avec le prfixe spcifi.
 @since Release 1.0
*/
BOOL TString_startsWith0 (TString* this, TString* string, long offset) {
	
	BOOL result = FALSE;
	long i;
	
	if (string->length(string) <= this->length(this) - offset) {
		result = TRUE;
		for (i=offset; i<(string->length(string) + offset) && result; i++) {
			if (this->string[i] != string->string[i - offset]) {
				result = FALSE;
			}
		}
	}
	
	return result;
}

/**
 * Cette mthode vrifie si la chane de caractres commence avec le prfixe spcifi.
 @param prefix Le prfixe.
 @return result - Vrai si la chane de caractres commence avec le prfixe spcifi.
 @since Release 1.0
*/
BOOL TString_startsWith1 (TString* this, TString* string) {
	
	BOOL result;
	
	result = this->startsWith0(this, string, 0);
	
	return result;
}

/**
 * Cette mthode retourne la sous-chane de caractres dlimit par les index spcifis.
 @param beginIndex L'index o commence la sous-chane.
 @param endIndex L'index o finit la sous-chane.
 @return substring - La sous-chane de caractres.
 @since Release 1.0
*/
TString* TString_substring0	(TString* this, long beginIndex, long endIndex) {
	
	TString *substring;
	
	long length = this->length(this) - beginIndex - (this->length(this) - endIndex - 1);
	char region[length];
	
	this->getChars(this, beginIndex, beginIndex + length - 1, region, 0);
	
	substring = TString5(region, length);
	
	return substring;
}

/**
 * Cette mthode retourne la sous-chane de caractres commenant par l'index spcifi.
 @param beginIndex L'index o commence la sous-chane.
 @return substring - La sous-chane de caractres.
 @since Release 1.0
*/
TString* TString_substring1	(TString* this, long beginIndex) {

	TString *substring;
	
	substring = this->substring0(this, beginIndex, this->length(this) - 1);
	
	return substring;
}

/**
 * Cette mthode retourne un vecteur de caractres compos des caractres de la chane de
 * caractres.
 @return array - Le vecteur de caractres.
 @since Release 1.0
*/
char* TString_toCharArray (TString* this) {
	
	long i;
	char *array;
	
	array = (char*)malloc(sizeof(char) * this->length(this));
	
	for (i=0; i<this->length(this); i++) {
		array[i] = this->string[i];
	}
		
	return array;
}

/**
 * Cette mthode enlve les espaces des deux cts de la chane de caractres.
 @return string - La chane modifie.
 @since Release 1.0
*/
TString* TString_trim (TString* this) {
	
	long i;
	long ptrBegin = 0;
	long ptrEnd   = 0;
	TString *string;
	BOOL begin    = TRUE;
	BOOL end      = TRUE;
	
	string = this->clone(this);
	
	for (i=0; i<this->length(this) && begin; i++) {
		if (this->string[i] != ' ') {
			begin 	 = FALSE;
			ptrBegin = i;
		}
	}
	
	for (i=(this->length(this) - 1); i>=0 && end; i--) {
		if (this->string[i] != ' ') {
			end    = FALSE;
			ptrEnd = i;
		}
	}

	string = this->substring0(this, ptrBegin, ptrEnd);
	
	return string;
}
