/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TVector - Type Vector

Hirarchie:
TObject -> TVector
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TVector reprsente un vecteur pouvant tre modifi dynamiquement.
 @since Release 1.0
*/

//Structure de l'objet de Type Vector extension de TObject
typedef struct Vector {
	char *class;          //Le nom de la classe
	char *super;          //Le nom de la superclasse
	struct Vector* 	(*clone) 		(struct Vector*);
	BOOL 						(*equals) 	(struct Object*, struct Object*);
	void 						(*finalize) (struct Vector*);
	TString*				(*toString)	(struct Vector*);
	long size;         //La grosseur du tableau.
	TNode *firstNode;  //Le premier lment
	void 						(*addElement) 				(struct Vector*, void*);
	BOOL 						(*contains) 					(struct Vector*, void*);
	void						(*copyInto)						(struct Vector*, void*);
	void* 					(*elementAt) 					(struct Vector*, long);
	void* 					(*firstElement) 			(struct Vector*);
	long 						(*indexOf0) 					(struct Vector*, void*, long);
	long 						(*indexOf1) 					(struct Vector*, void*);
	void 						(*insertElementAt) 		(struct Vector*, void*, long);
	BOOL 						(*isEmpty) 						(struct Vector*);
	void* 					(*lastElement) 				(struct Vector*);
	long 						(*lastIndexOf0)				(struct Vector*, void*, long);
	long 						(*lastIndexOf1) 			(struct Vector*, void*);
	void 						(*removeAllElements) 	(struct Vector*);
	void 						(*removeElement)			(struct Vector*, void*);
	void 						(*removeElementAt) 		(struct Vector*, long);
	void 						(*setElementAt) 			(struct Vector*, void*, long);
} TVector;

//Dclaration des constructeurs
TVector* TVector0	();

//Dclaration des mthodes
void 			TVector_addElement 				(TVector*, void*);
BOOL 			TVector_contains 					(TVector*, void*);
void			TVector_copyInto					(TVector*, void*);
TVector* 	TVector_clone 						(TVector*);
void* 		TVector_elementAt 				(TVector*, long);
void 			TVector_finalize 					(TVector*);
void* 		TVector_firstElement 			(TVector*);
long 			TVector_indexOf0 					(TVector*, void*, long);
long 			TVector_indexOf1 					(TVector*, void*);
void 			TVector_insertElementAt 	(TVector*, void*, long);
BOOL 			TVector_isEmpty 					(TVector*);
void* 		TVector_lastElement 			(TVector*);
long 			TVector_lastIndexOf0			(TVector*, void*, long);
long 			TVector_lastIndexOf1 			(TVector*, void*);
void 			TVector_removeAllElements (TVector*);
void 			TVector_removeElement			(TVector*, void*);
void 			TVector_removeElementAt 	(TVector*, long);
void 			TVector_setElementAt 			(TVector*, void*, long);
TString*	TVector_toString					(TVector*);

/**
 * Ce constructeur instancie un objet de Type Vector.
 @since Release 1.0
*/
TVector* TVector0 () {
	TVector *this = (TVector*)malloc(sizeof(TVector));

	this->class = "TVector";
	this->super = "TObject";
	
	this->size = 0;
	this->firstNode = NULL;
	
	this->clone    = TVector_clone;
	this->equals   = TObject_equals;
	this->finalize = TVector_finalize;
	this->toString = TVector_toString;

	this->addElement 	 			= TVector_addElement;
	this->contains 					= TVector_contains;
	this->copyInto 					= TVector_copyInto;
	this->elementAt					= TVector_elementAt;
	this->firstElement 			= TVector_firstElement;
	this->indexOf0 					= TVector_indexOf0;
	this->indexOf1 					= TVector_indexOf1;
	this->insertElementAt 	= TVector_insertElementAt;
	this->isEmpty 					= TVector_isEmpty;
	this->lastElement 			= TVector_lastElement;
	this->lastIndexOf0 			= TVector_lastIndexOf0;
	this->lastIndexOf1 			= TVector_lastIndexOf1;
	this->removeAllElements = TVector_removeAllElements;
	this->removeElement			= TVector_removeElement;
	this->removeElementAt 	= TVector_removeElementAt;
	this->setElementAt 			= TVector_setElementAt;

	return this;
}

/**
 * Cette mthode indique si le vecteur est vide.
 @return result - Vrai si le vecteur est vide.
 @since Release 1.0
*/
BOOL TVector_isEmpty (TVector* this) {
	
	BOOL result = FALSE;
	
	if (this->size > 0) {
		result = TRUE;
	}
	
	return result;
}

/**
 * Cette mthode d'enlever tous les lments du vecteur.
 @since Release 1.0
*/
void TVector_removeAllElements (TVector* this) {

	long i;
	long nbrElements = this->size;
	
	for (i=0; i<nbrElements; i++) {
		this->removeElementAt(this, 0);
	}
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TVector_finalize (TVector* this) {
	
	this->removeAllElements(this);
	
	TRY
		free(this);
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode permet d'enlever l'lment du vecteur  l'index spcifi.
 @param index L'index o se trouve l'lment.
 @since Release 1.0
*/
void TVector_removeElementAt (TVector* this, long index) {
	
	long i;
	TNode *remNode;
	TNode *refNode;
	TNode *fstNode;
	
	if (index < this->size && index >= 0) {
		fstNode = TNode1(NULL, this->firstNode);
		refNode = fstNode;
		for (i=0; i<index; i++) {
			refNode = refNode->after;
		}
		remNode = refNode->after;
		refNode->after = remNode->after;
		if (index == 0) {
			this->firstNode = remNode->after;
		}		
		remNode->finalize(remNode);
		this->size = this->size - 1;
		fstNode->finalize(fstNode);
	}
}

/**
 * Cette mthode permet d'ajouter un lment au vecteur.  Ce dernier est ajout  l'index spcifi.
 @param element L'objet  ajouter au vecteur.
 @param index L'index o faire l'ajout.
 @since Release 1.0
*/
void TVector_insertElementAt (TVector* this, void* element, long index) {
	
	long i;
	TNode *newNode;
	TNode *refNode;
	TNode *fstNode;
	
	if (index <= this->size && index >= 0) {
		newNode = TNode0(element);
		fstNode = TNode1(NULL, this->firstNode);
		refNode = fstNode;
		for (i=0; i<index; i++) {
			refNode = refNode->after;
		}
		newNode->after = refNode->after;
		refNode->after = newNode;
		if (index == 0) {
			this->firstNode = newNode;
		}
		this->size = this->size + 1;
		fstNode->finalize(fstNode);
	}
}

/**
 * Cette mthode permet d'ajouter un lment au vecteur.  Ce dernier est ajout  la fin du vecteur.
 @param element L'objet  ajouter au vecteur.
 @since Release 1.0
*/
void TVector_addElement (TVector* this, void* element) {
	
	this->insertElementAt(this, element, this->size);
}

/**
 * Cette mthode retourne l'lment prsent  la position de l'index spcifi.
 @param index L'index o se trouve l'lment.
 @return element - L'lment prsent  l'index spcifi.
 @since Release 1.0
*/
void* TVector_elementAt (TVector* this, long index) {

	long i;
	void* element = NULL;
	TNode *node;
	
	if (index >= 0 && index < this->size) {	
		node = this->firstNode;
		for (i=0; i<index; i++) {
			node = node->after;
		}
	
		element = node->element;
	}

	return element;
}

/**
 * Cette mthode retourne le premier lment du vecteur.
 @return element - Le premier lment du vecteur.
 @since Release 1.0
*/
void* TVector_firstElement (TVector* this) {

	void* element = NULL;
	
	this->elementAt(this, 0);
	
	return element;
}

/**
 * Cette mthode retourne le dernier lment du vecteur.
 @return element - Le dernier element du vecteur.
 @since Release 1.0
*/
void* TVector_lastElement (TVector* this) {

	void* element = NULL;
	
	this->elementAt(this, this->size);
	
	return element;
}

/**
 * Cette mthode retourne un objet de Type Vector qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TVector* TVector_clone (TVector* this) {
	
	long i;
	TVector *clone;
	
	clone = TVector0();
	
	for (i=0; i<this->size; i++) {
		clone->addElement(clone, this->elementAt(this, i));
	}
	
	return clone;
}

/**
 * Cette mthode retourne l'index de la premire occurence de l'lment rencontre
 * dans le vecteur  partir de l'index spcifi.
 @param element L'lment  rechercher.
 @return fromIndex - L'index d'o commencer.
 @return index - L'index de la premire occurence.
 @since Release 1.0
*/
long TVector_indexOf0 (TVector* this, void* element, long fromIndex) {
	
	long i;
	long index = -1;
	BOOL trouve = FALSE;
	
	for (i=fromIndex; i<this->size && !trouve; i++) {
		if (this->elementAt(this, i) == element) {
			trouve = TRUE;
			index = i;
		}
	}
	
	return index;
}

/**
 * Cette mthode retourne l'index de la premire occurence de l'lment rencontre
 * dans le vecteur.
 @param element L'lment  rechercher.
 @return index - L'index de la premire occurence.
 @since Release 1.0
*/
long TVector_indexOf1 (TVector* this, void* element) {

	long index;
	
	index = this->indexOf0(this, element, 0);

	return index;
}

/**
 * Cette mthode indique si l'lment spcifi se trouve dans le vecteur.
 @param element L'lment  rechercher.
 @return result - Vrai si l'lment se trouve dans le vecteur.
 @since Release 1.0
*/
BOOL TVector_contains (TVector* this, void* element) {

	long i;
	BOOL result = FALSE;

	for (i=0; i<this->size && !result; i++) {
		if (this->elementAt(this, i) == element) {
			result = TRUE;
		}
	}

	return result;
}

/**
 * Cette mthode retourne l'index de la dernire occurence de l'lment rencontre
 * dans le vecteur en cherchant  reculon  partir de l'index spcifi.
 @param element L'lment  rechercher.
 @return fromIndex - L'index d'o commencer.
 @return index - L'index de la dernire occurence.
 @since Release 1.0
*/
long TVector_lastIndexOf0 (TVector* this, void* element, long fromIndex) {

	long i;
	long index = -1;
	BOOL trouve = FALSE;
	
	if (fromIndex < this->size && fromIndex >= 0) {
		for (i=fromIndex; i>=0 && !trouve; i--) {
			if (this->elementAt(this, i) == element) {
				trouve = TRUE;
				index = i;
			}
		}
	}
	
	return index;
}

/**
 * Cette mthode retourne l'index de la dernire occurence de l'lment rencontre.
 @param element L'lment  rechercher.
 @return index - L'index de la dernire occurence.
 @since Release 1.0
*/
long TVector_lastIndexOf1 (TVector* this, void* element) {

	long index;
	
	index = this->lastIndexOf0(this, element, this->size - 1);

	return index;
}

/**
 * Cette mthode copie les lment du vecteur vers le vecteur spcifi.
 @param anArray Le vecteur cible.
 @since Release 1.0
*/
void TVector_copyInto (TVector* this, void* anArray) {
	
	long i;
	void *array[this->size];
	
	*array = anArray;
	
	for (i=0; i<this->size; i++) {
		array[i] = (TObject*)this->elementAt(this, i);
	}
}

/**
 * Cette mthode remplace l'lment  l'index spcifi.
 @param element L'lment de rechange.
 @param index L'index o effectuer le remplacement.
 @since Release 1.0
*/
void TVector_setElementAt (TVector* this, void* element, long index) {
	
	this->removeElementAt(this, index);
	this->insertElementAt(this, element, index);
}

/**
 * Cette mthode enlve du vecteur la premire occurence de l'lment spcifi.
 @param element L'lment  enlever.
 @since Release 1.0
*/
void TVector_removeElement (TVector* this, void* element) {
	
	long index;
	
	index = this->indexOf1(this, element);
	
	if (index >= 0) {
		this->removeElementAt(this, index);
	}
}

/**
 * Cette mthode retourne une chane de caractres reprsentant l'objet.
 * Pour la classe TVector, il s'agit du nombre d'lments, suivi de la
 * reprsentation de chaqu'un.
 @return string - La chane de caractre reprsentant l'objet.
 @since Release 1.0
*/
TString* TVector_toString	(TVector* this) {
	
	long i;
	TString *string;
	
	string = Long_toString(this->size);
	
	string = string->concat(string, s("["));
	
	for (i=0; i<this->size; i++) {
		string = string->concat(string, s("["));
		string = string->concat(string, ((TObject*)this->elementAt(this, i))->toString(this->elementAt(this, i)));
		string = string->concat(string, s("]"));
	}
	
	string = string->concat(string, s("]"));
	
	return string;
}
